@@ -1,21 +1,31 @@
-"cohen.d" <- function(x,group,alpha=.05,std=TRUE,dictionary=NULL) {
+"cohen.d" <- function(x,group,alpha=.05,std=TRUE,sort=NULL,dictionary=NULL,MD=TRUE) {
 cl <- match.call()
+group2 <- NULL
+if(inherits(x,"formula")) {ps <- fparse(x)   #group was specified, call describeBy
+	x <- get(ps$y)
+	group <- ps$x 
+	if(length(group ) > 1) {group2 <- group[2]
+	 group <- group[1]}
+	 }
+	 
+	if(!is.null(group2)){cohen.d.by(x =x, group=group,group2=group2)} else {
+	x <- char2numeric(x)  # just in case there are character values  12/12/20
 if ((length(group) ==1) && ( group %in% colnames(x) )) {group <- which(colnames(x) %in% group)
-  group.in <- TRUE}
+  group.in <- TRUE} else {group.in <- FALSE}
  stats <- statsBy(x,group)
  S <- stats$rwg
 
- S.inv <- solve(S)
-
+ S.inv <- Pinv(S)   #the pseudo inverse because it is possible this is not PSD   added December 15, 2019
 d <- stats$mean[2,] - stats$mean[1,]
 sd.p <- sqrt((( (stats$n[1,]-1) * stats$sd[1,]^2) + (stats$n[2,]-1) * stats$sd[2,]^2)/(stats$n[1,]+stats$n[2,])) #if we subtract 2 from n, we get Hedges g
 sd.ph <- sqrt((((stats$n[1,]-1) * stats$sd[1,]^2) + (stats$n[2,]-1) * stats$sd[2,]^2)/(stats$n[1,]+stats$n[2,]-2)) #if we subtract 2 from n, we get Hedges g
 n <- stats$n[1,]+ stats$n[2,]
 cohen.d <- d/sd.p
- d <- cohen.d    #basically use this in the Mahalanobis distance
+
 hedges.g <- d/sd.ph
+d <- cohen.d    #basically use this in the Mahalanobis distance
 names(cohen.d) <- colnames(x)
-if(group.in) {
+if(!group.in) {group <- which(colnames(stats$n)=="group")}
 n <- n[-group]
 d <- d[-group]
 n1 <- stats$n[1,-group]
@@ -25,18 +35,25 @@ p2 <- n2/n
 cohen.d <- cohen.d[-group]
 hedges.g <- hedges.g[-group]
 
-r <- cohen.d/sqrt(cohen.d^2 + 1/(  p1*p2)) } else {r <- cohen.d/sqrt(cohen.d^2 + 1/( p1*p2) )} #for unequal n otherwise this is just 4
+r <- cohen.d/sqrt(cohen.d^2 + 1/(  p1*p2)) #} else {r <- cohen.d/sqrt(cohen.d^2 + 1/( p1*p2) )} #for unequal n otherwise this is just 4
 t <- d2t(cohen.d,n)
 p <- ( 1-pt(abs(t),n-2)) * 2
-D <- sqrt(t(d) %*% S.inv %*% d)
-
-D <- as.vector(D)
+if(MD) {D <- sqrt(t(d) %*% S.inv %*% d)   #convert to D units from D2 units
+wt.d <- t(d) %*% S.inv *d    #what is this?
+D <- as.vector(D)} else {D <- NA}
 cohen.d.conf <- cohen.d.ci(cohen.d,n1=n1,n2=n2,alpha=alpha)
-if(!is.null(dictionary)) {dict = dictionary[colnames(x),]} else {dict=NULL}
-result <- list(cohen.d = cohen.d.conf,hedges.g = hedges.g,M.dist = D, r=r,t=t,n=n,p=p, descriptive=stats,dict=dict,Call=cl)
+if(!is.null(dictionary)) {dict <- dictionary[match(rownames(cohen.d.conf),rownames(dictionary)),,drop=FALSE]
+   cohen.d.conf <- cbind(cohen.d.conf,dict)} else {dict=NULL}
+if(!is.null(sort)) {if(sort %in%( c("decreasing","descending","TRUE"))) {ord <- order(cohen.d.conf["effect"],decreasing=TRUE)} else {ord <- order(cohen.d.conf["effect"],decreasing=FALSE)}
+       cohen.d.conf <- cohen.d.conf[ord,]
+       dict <- dict[ord,,drop=FALSE]
+      }
+se <- (cohen.d.conf[,3] - cohen.d.conf[,1])/2  #average upper - lower 
+result <- list(cohen.d = cohen.d.conf,hedges.g = hedges.g, M.dist = D, r=r,t=t,n=n,p=p, wt.d =wt.d,descriptive=stats,se=se,dict=dict,Call=cl)
 class(result) <- c("psych","cohen.d")
 return(result)
 }
+}
 
 "d2t" <- function(d,n=NULL,n2=NULL,n1=NULL) {if(is.null(n1)) {t <- d*sqrt(n)/2} else 
     if(is.null(n2)) {t <- d*sqrt(n1) } else {
@@ -56,10 +73,10 @@ return(result)
 
  upper <- try(t2d( uniroot(function(x) {suppressWarnings(pt(q=t[i],df=nmax[i]-2,ncp=x)) - alpha/2}, c(min(-5,-abs(t[i])*10),max(5,abs(t[i])*10)))$root,n=n[i],n2=n2[i],n1=n1[i]),silent=TRUE)
   
-     if(class( upper)=="try-error") {ci[i,3] <- NA} else {ci[i,3] <- upper}
+     if(inherits( upper, "try-error")) {ci[i,3] <- NA} else {ci[i,3] <- upper}
     ci[i,2] <- d[i]
   lower.ci  <- try(t2d(uniroot(function(x) {suppressWarnings(pt(q=t[i],df=nmax[i]-2,ncp=x)) - tail}, c(min(-5,-abs(t[i])*10),max(5,abs(t[i]) *10)))$root,n=n[i],n2=n2[i],n1=n1[i]),silent=TRUE)
-    if(class( lower.ci)=="try-error") {ci[i,1] <- NA} else {ci[i,1] <- lower.ci}
+    if(inherits( lower.ci,"try-error")) {ci[i,1] <- NA} else {ci[i,1] <- lower.ci}
    }
    colnames(ci) <- c("lower","effect","upper")
    rownames(ci) <- names(d)
@@ -88,7 +105,12 @@ invisible(result) #return the values as well
   
  
  "cohen.d.by" <- 
-function(x,group,group2,alpha=.05)  {
+function(x,group,group2,alpha=.05,MD=TRUE)  {
+if(inherits(x,"formula")) {ps <- fparse(x)   #group was specified, call describeBy
+	x <- get(ps$y)
+	group <- ps$x 
+	if(length(group ) > 1) {group2 <- group[2]
+	 group <- group[1]}}
 
 
   group1 <- group
@@ -101,22 +123,23 @@ function(x,group,group2,alpha=.05)  {
     for(i in 1:length(categories)) {
       group <- subset(x,x[group2]==categories[i])
       group <- group[-group2]
-      result[[i]] <- cohen.d(group,group1name)     
+      result[[i]] <- cohen.d(group,group1name,MD=MD)     
       }
       names(result) <- paste0(group1name,"for",group2name ,categories)
       class(result) <- class(result) <- c("psych","cohen.d.by")
       return(result)
-   }
+}
     
     
     "print.cohen.d" <- function(x,digits=2) {cat("Call: ")
-            print(x$Call)
+           print(x$Call)
             cat("Cohen d statistic of difference between two means\n")
-            print(x$cohen.d,digits=digits)
+            if(NCOL(x$cohen.d) ==  3)  {print(round(x$cohen.d,digits=digits))} else {print( data.frame(round(x$cohen.d[1:3],digits=digits),x$cohen.d[4:NCOL(x$cohen.d)]))}
+          
             cat("\nMultivariate (Mahalanobis) distance between groups\n")
             print(x$M.dist,digits=digits) 
-            cat("r equivalent lof difference between two means\n")
-            print(x$r,digits=digits)
+            cat("r equivalent of difference between two means\n")
+            print(round(x$r,digits=digits))
             }
  
     "print.cohen.d.by" <- function(x,digits=2) {cat("Call: ")
