@@ -7,7 +7,7 @@
 function() {}
 
 "lowerMat" <- 
-function(R,digits=2) {
+function(R,digits=2,minlength=5) {
    lowleft <- lower.tri(R,diag=TRUE)
    nvar <- ncol(R)
 	nc <- digits+3
@@ -15,7 +15,7 @@ function(R,digits=2) {
 	k1 <- width/(nc+2)
    if(is.null(colnames(R))) {colnames(R) <- paste("C",1:nvar,sep="")}
    if(is.null(rownames(R))) {rownames(R) <- paste("R",1:nvar,sep="")}
-	colnames(R) <- abbreviate(colnames(R),minlength=digits+3)
+	colnames(R) <- abbreviate(colnames(R),minlength=minlength)
 	
 	nvar <- ncol(R)
 	nc <- digits+3
@@ -31,9 +31,9 @@ function(R,digits=2) {
 	invisible(R[lower.tri(R,diag=FALSE)])}
 	
 "lowerCor" <- 
-function(x,digits=2,use="pairwise",method="pearson") {
+function(x,digits=2,use="pairwise",method="pearson",minlength=5) {
    R <- cor(x,use=use,method=method)
-   lowerMat(R,digits)
+   lowerMat(R,digits,minlength=minlength)
    invisible(R)
    }
 
@@ -235,14 +235,14 @@ return(y)}
  #use lookup to take fa/ic output and show the results 
  #modified July 4, 2017 to allow for omega output as well
  #modified June 23, 2018 to limit to top n items per factor and abs(loading) > cut
+ #modified April 30, 2020 to include the ability to handle bassAckward output
 "fa.lookup"  <-
    function(f,dictionary=NULL,digits=2,cut=.0,n=NULL,sort=TRUE) {
-    if(sort) {f <- fa.sort(f)}
-    none <- NA 
-   if(length(class(f)) > 1){ obnames <- cs(omega, fa, principal, iclust, none)
+    omega <- bassAck <- none <- lavaan <- NA    #weird requirement to avoid being global
+   if(length(class(f)) > 1){ obnames <- cs(omega, fa, principal, iclust,bassAck, none)
      value <- inherits(f, obnames, which=TRUE)
 			   if (any(value > 1)) {value <- obnames[which(value >0)]} else {value <- "none"}}
-  
+   if(sort & (value !="bassAck")) {f <- fa.sort(f)}
    switch(value,
     
     omega = {f <- f$schmid$sl
@@ -257,6 +257,14 @@ return(y)}
               f <- f$loading},
     iclust = {f <- f$loadings
               h2 <- NULL},
+    bassAck = {nlevels <- length(f$bass.ack)	
+              h2 <- NULL
+              ord1 <- rownames(fa.sort(f$bass.ack[[nlevels-1]]))
+               f <- f$bass.ack[[nlevels]]
+               f <- f[,ord1]
+               f <- fa.sort(f)
+               colnames(f) <- paste0("F",1:ncol(f))           
+    },
     none = {f <- f
             h2 <- NULL})
     
@@ -303,6 +311,7 @@ return(results)}
  #revised 12/07/18 to allow for simple matrix input
  #read a matrix, return a matrix
  #read a list, return a list
+ #modified 9/2/20 for the case of not full rank cluster scores
   "fa.organize" <- 
 function(fa.results,o=NULL,i=NULL,cn=NULL,echelon=TRUE,flip=TRUE) {
 if(is.null(names(fa.results)) )  {temp <- fa.results   #the matrix form          
@@ -313,22 +322,32 @@ if(is.null(names(fa.results)) )  {temp <- fa.results   #the matrix form
                  temp <- t( t(temp) * flipper ) }
                  if(!is.null(o)) {temp <- temp[,o]}
                  if(!is.null(i)) {temp <-temp[i,]}
-                 fa.results <- temp } else { # the list form 
+                 fa.results <- temp 
+                 nf <- ncol(temp)} else { # the list form 
+    nf <- ncol(fa.results$loadings)
      if(echelon & is.null(o) ) {temp <- apply(abs(  fa.results$loadings),1,which.max)
      nf <- ncol(fa.results$loadings)
      nvar <- nrow(fa.results$loadings)
-  o <- 1:nf
+  o <- rep(NA,nf)
   k <- 1
   o[k] <- temp[k]
   for (ki in 2:nvar) {if (!(temp[ki] %in% o[1:k])) {o[k+1] <- temp[ki]
     k <- k + 1}  }
     }
+    
+    #now, a kludge for the case where there are some empty columns
+   
+    n.good <- sum(1:nf %in% o)
+    if (n.good < nf) { tt <- 1:nf
+      o[(n.good + 1):nf] <- tt[!1:nf %in% o]
+       }
   if(flip) {
         total.load <- colSums(fa.results$loadings)
        flipper <- sign(total.load)
         flipper[flipper==0] <-1 } else { flipper <- rep(1,NCOL(fa.results$loadings)) }
    fa.results$loadings <- t(t(fa.results$loadings) * flipper)                      
 
+
   if(!is.null(o)) {fa.results$loadings <- fa.results$loadings[,o]
       flipper <- flipper[o] 
        fa.results$Structure <- t(t(fa.results$Structure[,o]) * flipper)
@@ -466,7 +485,11 @@ result
 }
 
 
-
+"lookupItems" <- function(content=NULL,dictionary=NULL,search=c("Item","Content","item")) {
+  location <-which(colnames(dictionary) %in% search)
+  value <- grep(content,dictionary[,location])
+  dictionary[value,,drop=FALSE]
+}
 
 
 "falsePositive" <- function(sexy=.1,alpha=.05,power=.8) {
@@ -554,6 +577,8 @@ text(x,y-2,"Unreliable but Valid")
 # print(round(R,digits))
 # invisible(R)}
 
+#should we add in the abilty to not choke on character variables?
+
 "cor2" <- function(x,y=NULL,digits=2,use="pairwise",method="pearson") {
 multi <- FALSE
 if(is.list(x) && is.null(y)) {multi <- TRUE
@@ -585,21 +610,34 @@ pretty}
 
 
 #October 25, 2016
+#June 18 2020  added  as.factor to convert character strings that are not stored as factors
+#this has a downsize that it converts numbers stored as characters to factors (see nchar2numeric)
 "char2numeric" <- function(x) {
  nvar <- NCOL(x)
  for(i in 1:nvar) {   
         if(!is.numeric(x[[i]] ))  {
-                                  if(is.factor(unlist(x[[i]])) | is.character(unlist(x[[i]]))) {  x[[i]] <- as.numeric(x[[i]]) 
+                                  if(is.factor(unlist(x[[i]])) | is.character(unlist(x[[i]]))) {  x[[i]] <- as.numeric(as.factor(x[[i]])) 
                           } else {x[[i]] <- NA} }
               } 
               invisible(x)} 
+#added June 25, 2020 to handle the case of numeric data stored as characters              
+"nchar2numeric" <- function(x) {
+ nvar <- NCOL(x)
+ for(i in 1:nvar) {   
+        if(!is.numeric(x[[i]] ))  { 
+                if(is.factor(unlist(x[[i]])) | is.character(unlist(x[[i]]))) {
+                                  if(is.factor(unlist(x[[i]]))) {  x[[i]] <- as.numeric(as.factor(x[[i]])) } else {x[[i]] <- as.numeric(x[[i]])}
+                          } else {x[[i]] <- NA} }
+              } 
+              invisible(x)} 
+
 
 #this just shows if it is a matrix is symmetric and has diagonals of 1
 "isCorrelation" <-  function(x) {value <- FALSE
   if(NROW(x) == NCOL(x)) {
   if( is.data.frame(x)) {if(isSymmetric(unname(as.matrix(x)))) { value <- TRUE}} else {if(isSymmetric(unname(x))) {value <- TRUE}}}
   value <- value && isTRUE(all.equal(prod(diag(as.matrix(x))),1) )
-  value <- value && isTRUE((min(x)>= -1) & (max(x) <= 1))
+  value <- value && isTRUE((min(x,na.rm=TRUE)>= -1) & (max(x,na.rm=TRUE) <= 1))  
   return(value)}
   
   #this just shows if it is a symmetric matrix
@@ -627,7 +665,11 @@ fromTo <- function(data,from,to=NULL) {cn <- colnames(data)
   
   
 
-
+#flip -- not public but useful trick
+flip <- function(R,key) {#reverse correlations with keys  < 0
+	R[key<0,] <- -R[key < 0,]
+	R[,key<0] <- -R[,key < 0]
+  return(R)}
     
     
  
