@@ -4,6 +4,7 @@
 
 "print.psych" <-
 function(x,digits=2,all=FALSE,cut=NULL,sort=FALSE,short=TRUE,lower=TRUE,signif=NULL,...) { 
+#probably need to fix this with inherits but trying to avoid doing that now.
 
 if(length(class(x)) > 1)  { value <- class(x)[2] } else {
 #these next test for non-psych functions that may be printed using print.psych.fa
@@ -30,10 +31,12 @@ esem  = {print.psych.esem(x,digits=digits,short=short,cut=cut,...)},
   cta = {print.psych.cta(x,digits=digits,all=all,...)},
   mediate = {print.psych.mediate(x,digits=digits,short=short,...)},
   multilevel = {print.psych.multilevel(x,digits=digits,short=short,...)},
+  testRetest = {print.psych.testRetest(x,digits=digits,short=short,...)},
+  bestScales = {print.psych.bestScales(x,digits=digits,short=short,...)},
 
 ##Now, for the smaller print jobs, just do it here.
 all=  {class(x) <- "list"
-         print(x) },    #find out which function created the data and then print accordingly
+         print(x,digits=digits) },    #find out which function created the data and then print accordingly
          
 alpha = {
 	cat("\nReliability analysis ",x$title," \n")
@@ -59,14 +62,60 @@ autoR = {cat("\nAutocorrelations \n")
        print(round(x$autoR,digits=digits))
        
  },
+ 
+ bassAck = {
+   cat("\nCall: ")
+   print(x$Call)
+nf <- length(x$bass.ack)-1
+for (f in 1:nf) {
+cat("\n",f, 
+x$sumnames[[f]])}
 
-bestScales ={ 
+if(!short) {
+for (f in 1:nf) {
+cat("\nFactor correlations\n ")
+print(round(x$bass.ack[[f]],digits=digits))}
+} else {cat("\nUse print with the short = FALSE option to see the correlations, or use the summary command.")}
+ },
+ 
+ 
+ auc = {cat('Decision Theory and Area under the Curve\n')
+cat('\nThe original data implied the following 2 x 2 table\n')
+print(x$probabilities,digits=digits)
+cat('\nConditional probabilities of \n') 
+print(x$conditional,digits=digits)
+cat('\nAccuracy = ',round(x$Accuracy,digits=digits),' Sensitivity = ',round(x$Sensitivity,digits=digits), '  Specificity = ',round(x$Specificity,digits=digits),  '\nwith Area Under the Curve = ', round(x$AUC,digits=digits) )
+cat('\nd.prime = ',round(x$d.prime,digits=digits), ' Criterion = ',round(x$criterion,digits=digits), ' Beta = ', round(x$beta,digits=digits))
+cat('\nObserved Phi correlation = ',round(x$phi,digits=digits), '\nInferred latent (tetrachoric) correlation  = ',round(x$tetrachoric,digits=digits)) 
+},
+
+bestScales = {if(!is.null(x$first.result)) {
+   cat("\nCall = ")
+   print(x$Call)
+   # print(x$first.result)
+   #  print(round(x$means,2))
+    print(x$summary,digits=digits)
+     # x$replicated.items
+      
+    items <- x$items
+     size <- NCOL(items[[1]])
+     nvar <- length(items)
+     for(i in 1:nvar) {
+     if(NCOL(items[[i]]) > 3) {items[[i]]  <- items[[i]][,-1]}
+   #  items[[i]][,2:3] <- round(items[[i]][,2:3],digits)
+   if(length( items[[i]][1]) > 0 )  {
+   items[[i]][,c("mean.r","sd.r")] <- round(items[[i]][,c("mean.r","sd.r")],digits)
+      }}
+
+     cat("\n Best items on each scale with counts of replications\n")
+     print(items)} else {
      df <- data.frame(correlation=x$r,n.items = x$n.items)
     cat("The items most correlated with the criteria yield r's of \n")
     print(round(df,digits=digits)) 
     if(length(x$value) > 0) {cat("\nThe best items, their correlations and content  are \n")
      print(x$value) } else {cat("\nThe best items and their correlations are \n")
      for(i in 1:length(x$short.key)) {print(round(x$short.key[[i]],digits=digits))} 
+     }  
      } 
       },
       
@@ -156,13 +205,14 @@ cluster.loadings =  {
   },
 
 cohen.d = {cat("Call: ")
-            print(x$Call)
+           print(x$Call)
             cat("Cohen d statistic of difference between two means\n")
-            print(x$cohen.d,digits=digits)
+            if(NCOL(x$cohen.d) ==  3)  {print(round(x$cohen.d,digits=digits))} else {print( data.frame(round(x$cohen.d[1:3],digits=digits),x$cohen.d[4:NCOL(x$cohen.d)]))}
+          
             cat("\nMultivariate (Mahalanobis) distance between groups\n")
             print(x$M.dist,digits=digits) 
             cat("r equivalent of difference between two means\n")
-            print(x$r,digits=digits)
+            print(round(x$r,digits=digits))
             },
 
 cohen.d.by = {cat("Call: ")
@@ -189,7 +239,11 @@ comorbid = {cat("Call: ")
    },
 
 
-
+corCi = {#cat("Call:")
+        # print(x$Call)
+                cat("\n Correlations and normal theory confidence intervals \n")
+                print(round(x$r.ci,digits=digits))
+},
      
 cor.ci = {cat("Call:")
               print(x$Call) 
@@ -219,7 +273,7 @@ corr.test = {cat("Call:")
               if(x$sym) {cat("Probability values (Entries above the diagonal are adjusted for multiple tests.) \n")} else {
                  if (x$adjust != "none" )  {cat("Probability values  adjusted for multiple tests. \n")}}
              print(round(x$p,digits))
-             cat("\n To see confidence intervals of the correlations, print with the short=FALSE option\n")
+            if(short)  cat("\n To see confidence intervals of the correlations, print with the short=FALSE option\n")
              if(!short) {cat("\n Confidence intervals based upon normal theory.  To get bootstrapped values, try cor.ci\n")
              if(is.null(x$ci.adj)) { ci.df <- data.frame(raw=x$ci) } else {
              ci.df <- data.frame(raw=x$ci,lower.adj = x$ci.adj$lower.adj,upper.adj=x$ci.adj$upper.adj)}
@@ -235,7 +289,7 @@ corr.p = {cat("Call:")
               if(x$sym) {cat("Probability values (Entries above the diagonal are adjusted for multiple tests.) \n")} else {
                  if (x$adjust != "none" )  {cat("Probability values  adjusted for multiple tests. \n")}}
              print(round(x$p,digits))
-             cat("\n To see confidence intervals of the correlations, print with the short=FALSE option\n")
+             if(short) cat("\n To see confidence intervals of the correlations, print with the short=FALSE option\n")
               if(!short) {cat("\n Confidence intervals based upon normal theory.  To get bootstrapped values, try cor.ci\n")
              print(round(x$ci,digits)) }
 
@@ -254,6 +308,17 @@ cor.wt = {cat("Weighted Correlations \n")
             print(x$Call)
         lowerMat(x$r,digits=digits) },
 
+crossV = {cat("Cross Validation\n")
+          cat("Call:")
+            print(x$Call)
+         cat("\nValidities from raw items and from the correlation matrix\n")
+         cat("Number of unique predictors used = ",x$nvars,"\n")
+        print(x$crossV,digits=digits)
+         cat("\nCorrelations based upon item based regressions \n")
+         lowerMat(x$item.R)
+          cat("\nCorrelations based upon correlation matrix based regressions\n")
+         lowerMat(x$mat.R)
+          },
          
 
 describe= {if(!is.null(x$signif)) {
@@ -282,6 +347,17 @@ describeData = {if  (length(dim(x))==1) {class(x) <- "list"
                  
             print(x$variables) }
          },
+         
+describeFast = {  cat("\n Number of observations = " , x$n.obs, "of which ", x$complete.cases,"  are complete cases.   Number of variables = ",x$nvar," of which ",x$numeric," are numeric and ",x$factors," are factors \n")
+                if(!short) {print(x$result.df) } else {cat("\n To list the items and their counts, print with short = FALSE") }
+                },
+direct = { cat("Call: ")
+        print(x$Call)
+        cat("\nDirect Schmid Leiman = \n")
+        print(x$direct,cut=cut)
+        } ,    
+
+        
  
 faBy = { cat("Call: ")
         print(x$Call)
@@ -289,7 +365,8 @@ faBy = { cat("Call: ")
         cat("\nAverage standardized loadings (pattern matrix) based upon correlation matrix for all cases as well as each group\n")
         cat("\nlow and high ", x$quant,"% quantiles\n")
         print(x$faby.sum,digits)
-        if(!short) {  
+        if(!short) { 
+        cat("\n Pooled loadings across groups \n") 
         print(x$mean.loading,digits=digits)
         cat("\n Average factor intercorrelations for all cases and  each group\n")
         print(x$mean.Phi,digits=2)
@@ -299,10 +376,25 @@ faBy = { cat("Call: ")
         print(x$Phi,digits=2)
         if(!is.null(x$fa)) {
         cat("\nFactor analysis results for each group\n")
-        print(x$faby.sum,digits)
-        }}
+        print(x$fa,digits)
+        } else {print("For a more informative output, print with short=FALSE")}}
                 
 },
+
+faCor = { cat("Call: ")
+        print(x$Call)
+       
+        if(!short) { cat("\n Factor Summary  for first solution\n")    
+        summary(x$f1)
+        cat("\n Factor Summary  for second solution\n") 
+                    summary(x$f2)  
+                    }
+      cat("\n Factor correlations between the two solutions\n")     
+       print(x$r,digits=digits)
+     
+     cat("\n Factor congruence between the two solutions\n")     
+       print(x$congruence,digits=digits)
+},
     
 guttman =  {
   cat("Call: ")
@@ -371,7 +463,10 @@ kappa = {if(is.null(x$cohen.kappa)) {
             print(x$confid,digits=digits)
             cat("\n Number of subjects =", x$n.obs,"\n")} else {
             cat("\nCohen Kappa (below the diagonal) and Weighted Kappa (above the diagonal) \nFor confidence intervals and detail print with all=TRUE\n")
-            print(x$cohen.kappa,digits) 
+            print(x$cohen.kappa,digits=digits) 
+            if(!is.null(x$av.kappa))  cat("\nAverage Cohen kappa for all raters ", 	round(x$av.kappa,digits=digits))
+            if(!is.null(x$av.wt))  cat("\nAverage weighted kappa for all raters ",round(x$av.wt,digits=digits))
+              
             }
    },
    
@@ -407,6 +502,29 @@ mixed= { cat("Call: ")
     if(lower) {if(length(x$rho)>1) { lowerMat (x$rho,digits=digits)} else {print(x$rho,digits)}}
    }},
    
+   
+omegaDirect ={ cat("Call: ")
+        print(x$Call)
+        cat("\nOmega from direct Schmid Leiman = ", round(x$omega.g,digits=digits),"\n")
+        print.psych.fa(x)
+        
+       eigenvalues <- diag(t(x$loadings) %*% x$loadings)
+       cat("\nWith eigenvalues of:\n")
+       print(eigenvalues,digits=2)
+         cat("The degrees of freedom for the model is",x$orth.f$dof," and the fit was ",round(x$orth.f$objective,digits),"\n")
+   	if(!is.na(x$orth.f$n.obs)) {cat("The number of observations was ",x$orth.f$n.obs, " with Chi Square = ",round(x$orth.f$STATISTIC,digits), " with prob < ", round(x$orth.f$PVAL,digits),"\n")}
+   	
+    if(!is.null(x$orth.f$rms)) {cat("\nThe root mean square of the residuals is ", round(x$orth.f$rms,digits),"\n") }
+     if(!is.null(x$orth.f$crms)) {cat("The df corrected root mean square of the residuals is ", round(x$orth.f$crms,digits),"\n") }
+    if(!is.null(x$orth.f$RMSEA)) {cat("\nRMSEA and the ",x$orth.f$RMSEA[4]  ,"confidence intervals are ",round(x$orth.f$RMSEA[1:3],digits+1))  }
+   	if(!is.null(x$orth.f$BIC)) {cat("\nBIC = ",round(x$orth.f$BIC,digits))}	
+
+        
+cat("\n Total, General and Subset omega for each subset\n")
+   colnames(x$om.group) <- c("Omega total for total scores and subscales","Omega general for total scores and subscales ", "Omega group for total scores and subscales")
+   #rownames(x$om.group) <- tn
+   print(round(t(x$om.group),digits))},  
+
  
  paired.r = {cat("Call: ")
             print(x$Call)
@@ -416,14 +534,36 @@ mixed= { cat("Call: ")
          cat("  With probability = ",round(x$p,digits))
          },
 
-  
-
+pairwise = {cat("Call: ")
+              print(x$Call) 
+              cat("\nMean correlations within/between scales\n")
+              lowerMat(x$av.r)
+              cat("\nPercentage of complete correlations\n")
+              lowerMat(x$percent)
+               cat("\nNumber of complete correlations per scale\n")
+              lowerMat(x$count)
+             if(!is.null(x$size)) {cat("\nAverage number of pairwise observations per scale\n")
+              lowerMat(round(x$size))}
+              cat("\n Imputed correlations (if found) are in the imputed object")
+              },
+pairwiseCounts = {cat("Call: ")
+              print(x$Call) 
+              cat("\nOverall descriptive statistics\n")
+              if(!is.null(x$description)) print(x$description)
+              cat("\nNumber of item pairs <=", x$cut," = ", dim(x$df)[1])
+              cat("\nItem numbers with pairs <= ",x$cut, " (row wise)", length(x$rows))
+              cat("\nItem numbers with pairs <= ",x$cut,"(col wise)", length(x$cols))
+              cat("\nFor names of the offending items, print with short=FALSE")
+              if(!short) {cat("\n Items names with pairs < ", x$cut," (row wise)\n", names(x$rows))
+                         cat("\n Items names with pairs <=",x$cut," (col wise)\n", names(x$cols))}
+              cat("\nFor even more details examine the rows, cols and df report" )
+    },
          
 parallel= {
 cat("Call: ")
               print(x$Call) 
               if(!is.null(x$fa.values) & !is.null(x$pc.values) ) {
-                  parallel.df <- data.frame(fa=x$fa.values,fa.sim =x$fa.sim,pc= x$pc.values,pc.sim =x$pc.sim)
+                  parallel.df <- data.frame(fa=x$fa.values,fa.sam =x$fa.simr,fa.sim=x$fa.sim,pc= x$pc.values,pc.sam =x$pc.simr,pc.sim=x$pc.sim)
 		fa.test <- x$nfact
 		pc.test <- x$ncomp
 		
@@ -431,7 +571,10 @@ cat("Call: ")
 		cat("the number of factors = ",fa.test, " and the number of components = ",pc.test,"\n")
                   cat("\n Eigen Values of \n")
                   
-                  colnames(parallel.df) <- c("Original factors","Simulated data","Original components", "simulated data")}
+                  colnames(parallel.df) <- c("Original factors","Resampled data", "Simulated data","Original components", "Resampled components", "Simulated components")
+                if(any(is.na(x$fa.sim))) parallel.df <- parallel.df[-c(3,6)]
+        
+                }
               if(is.na(fa.test) ) fa.test <- 0
               if(is.na(pc.test)) pc.test <- 0
                if(!any(is.na(parallel.df)))  {print(round(parallel.df[1:max(fa.test,pc.test),],digits))}  else {
@@ -447,7 +590,7 @@ cat("Call: ")
             }  
     },
          
-  partial.r = {cat("partial correlations \n")
+partial.r = {cat("partial correlations \n")
             print(round(unclass(x),digits))
          }, 
          
@@ -464,10 +607,12 @@ cat("Call: ")
             if(!is.null(x$twobytwo)) {
               print(x$twobytwo,digits=digits)
               cat("\n implies tetrachoric correlation of ",round(-x$rho,digits))} else {
-            
+            if(!isSymmetric(x$rho))  lower<- FALSE
             if(lower) {lowerMat (x$rho,digits) } else {print(x$rho,digits)}
+        
             cat("\n with tau of \n")
             print(x$tau,digits)
+            if(!is.null(x$tauy)) print(x$tauy,digits)
             }
    },
 
@@ -539,6 +684,8 @@ overlap =  {
     print(x$G6,digits)
   	cat("\nAverage item correlation:\n")
 	print(x$av.r,digits)
+	cat("\nMedian item correlation:\n")
+	print(x$med.r,digits)
 	cat("\nNumber of items:\n")
 	print(x$size)
 	cat("\nSignal to Noise ratio based upon average r and n \n")
@@ -555,7 +702,9 @@ overlap =  {
 	
 	  },
 	   
-                  
+frequency = { cat("Response frequencies (of non-missing items) \n")
+             print(unclass(x),digits=digits)
+            },                
 r.test =  {cat("Correlation tests \n")
             cat("Call:")
               print(x$Call)
@@ -565,8 +714,8 @@ r.test =  {cat("Correlation tests \n")
               if(!is.null(x$ci)) {cat("\n and confidence interval ",round(x$ci,digits) ) }
          },   
          
-residuals = { 
-   if (lower) {lowerMat (x,digits=digits)} else {print(x,digits)}
+residuals = { if(NCOL(x) == NROW(x)) {
+   if (lower) {lowerMat (x,digits=digits)}} else {print(round(unclass(x),digits))}  #tweaked 1/30/18
 	},   
 	
 	
@@ -591,6 +740,8 @@ scores =  {
 	print(x$alpha.ob,digits=digits)}
   	cat("\nAverage item correlation:\n")
   	print(x$av.r,digits=digits)
+  	cat("\nMedian item correlation:\n")
+  	print(x$med.r,digits=digits)
 	cat("\n Guttman 6* reliability: \n")
 	print(x$G6,digits=digits)   
 	cat("\nSignal/Noise based upon av.r : \n")
@@ -615,40 +766,35 @@ scores =  {
   },
   
 setCor= { cat("Call: ")
-              print(x$Call)
+                          print(x$Call)
             if(x$raw) {cat("\nMultiple Regression from raw data \n")} else {
             cat("\nMultiple Regression from matrix input \n")}
-           cat("\nBeta weights \n")
-           print(round(x$beta,digits))
-           cat("\nMultiple R \n") 
-           print(round(x$R,digits))
-            cat("multiple R2 \n") 
-            print(x$R2,digits)
-            cat("\nMultiple Inflation Factor (VIF) = 1/(1-SMC) = \n")
-            print(round(x$VIF,digits=digits))
-             cat("\n Unweighted multiple R \n") 
-           print(round(x$ruw,digits))
-             cat(" Unweighted multiple R2 \n") 
-           print(round(x$ruw^2,digits))
-              if(!is.null(x$se)) {
-               cat("\n SE of Beta weights \n")
-           print(round(x$se,digits))
-           cat("\n t of Beta Weights \n") 
-           print(round(x$t,digits))
-            cat("\nProbability of t < \n") 
-            print(signif(x$Probability,digits))
-            cat("\n Shrunken R2 \n")
-           print(x$shrunkenR2,digits)
-           cat("\nStandard Error of R2  \n") 
-           print(x$seR2,digits)
-            cat("\nF \n") 
-            print(round(x$F,digits))
-             cat("\nProbability of F < \n") 
-           print(signif(x$probF,digits+1))
-            cat("\n degrees of freedom of regression \n") 
-            print(x$df)
-           
-           }
+            if(!is.null(x$z)) cat("The following variables were partialed out:", x$z, "\n and are included in the calculation of df1 and df2\n")
+            ny <- NCOL(x$coefficients)
+          for(i in 1:ny) {cat("\n DV = ",colnames(x$coefficients)[i], "\n")
+       #   if(!is.na(x$intercept[i])) {cat(' intercept = ',round(x$intercept[i],digits=digits),"\n")}
+          if(!is.null(x$se)) {result.df <- data.frame( round(x$coefficients[,i],digits),round(x$se[,i],digits),round(x$t[,i],digits),signif(x$Probability[,i],digits),round(x$ci[,i],digits), round(x$ci[,(i +ny)],digits),round(x$VIF,digits))
+              colnames(result.df) <- c("slope","se", "t", "p","lower.ci","upper.ci",  "VIF")        
+              print(result.df) 
+               cat("\nResidual Standard Error = ",round(x$SE.resid[i],digits), " with ",x$df[2], " degrees of freedom\n")     
+              result.df <- data.frame(R = round(x$R[i],digits), R2 = round(x$R2[i],digits), Ruw = round(x$ruw[i],digits),R2uw =  round( x$ruw[i]^2,digits), round(x$shrunkenR2[i],digits),round(x$seR2[i],digits), round(x$F[i],digits),x$df[1],x$df[2], signif(x$probF[i],digits+1))
+              colnames(result.df) <- c("R","R2", "Ruw", "R2uw","Shrunken R2", "SE of R2", "overall F","df1","df2","p")
+              cat("\n Multiple Regression\n")
+             print(result.df)
+              } else {
+              result.df <- data.frame( round(x$coefficients[,i],digits),round(x$VIF,digits))
+              colnames(result.df) <- c("slope", "VIF")        
+              print(result.df)      
+              result.df <- data.frame(R = round(x$R[i],digits), R2 = round(x$R2[i],digits), Ruw = round(x$ruw[i],digits),R2uw =  round( x$ruw[i]^2,digits))
+              colnames(result.df) <- c("R","R2", "Ruw", "R2uw")
+              cat("\n Multiple Regression\n")
+             print(result.df)
+            
+             
+            
+              } 
+              }
+ 
             
             if(!is.null(x$cancor)) {
             cat("\nVarious estimates of between set correlations\n")
@@ -667,7 +813,6 @@ setCor= { cat("Call: ")
            
            }
 
-
    },
    
    
@@ -697,7 +842,12 @@ sim =  { if(is.matrix(x)) {x <-unclass(x)
    cat("\nGuttman lambda 6                          = ",round(x$lambda6,digits=digits))
    cat("\nAverage split half reliability            = ",round(x$meanr,digits=digits))
    cat("\nGuttman lambda 3 (alpha)                  = ",round(x$alpha,digits=digits))
+   cat("\nGuttman lambda 2                          = ", round(x$lambda2,digits=digits))
    cat("\nMinimum split half reliability  (beta)    = ",round(x$minrb,digits=digits))
+   if(x$covar) { cat("\nAverage interitem covariance = ",round(x$av.r,digits=digits)," with median = ", round(x$med.r,digits=digits))} else { cat("\nAverage interitem r = ",round(x$av.r,digits=digits)," with median = ", round(x$med.r,digits=digits))}
+   if(!is.na(x$ci[1])) {cat("\n                                            ",names(x$ci))
+   cat("\n Quantiles of split half reliability      = ",round(x$ci,digits=digits))}
+ 
       
  },
  
@@ -734,10 +884,12 @@ tau = {cat("Tau values from dichotomous or polytomous data \n")
             cat("tetrachoric correlation \n")
             if(!is.null(x$twobytwo)) {
               print(x$twobytwo,digits=digits)
-              cat("\n implies tetrachoric correlation of ",round(x$rho,digits))} else {
+              cat("\n implies tetrachoric correlation of ",round(x$rho,digits))} else {if(length(x$rho)>1) {
+            if(!isSymmetric(x$rho)) lower <- FALSE} else {lower<- FALSE}
            if(is.matrix(x$rho) &&  lower) {lowerMat (x$rho,digits)} else { print(x$rho,digits)}
             cat("\n with tau of \n")
             print(x$tau,digits)
+            if(!is.null(x$tauy)) print(x$tauy,digits)
           }
    },
    
@@ -762,16 +914,18 @@ KMO = {cat("Kaiser-Meyer-Olkin factor adequacy")
    print(round(x$MSAi,digits))
    },
    
-unidim = {cat("\nA measure of unidimensionality \n Call: ")
+ unidim=  {
+  cat("\nA measure of unidimensionality \n Call: ")
   print(x$Call)
   
   cat("\nUnidimensionality index = \n" )
   print(round(x$uni,digits=digits))
   
  cat("\nunidim adjusted index reverses negatively scored items.")
-  cat("\nalpha ","  Based upon reverse scoring some items.")
-  cat ("\naverage correlations are based upon reversed scored items") 
-   },
+ cat("\nalpha ","  Based upon reverse scoring some items.")
+ cat ("\naverage and median  correlations are based upon reversed scored items") 
+     },
+  
    
 yule = {cat("Yule and Generalized Yule coefficients")
      cat("\nCall: ")
